@@ -101,16 +101,18 @@ import Control.Monad.Fail( MonadFail )
 
 import           Control.Applicative
 import           Control.Exception
-import           Control.Monad (forM, forM_, replicateM_, unless, when)
+import           Control.Monad (forM, replicateM_, unless, when)
 import           Control.Monad.IO.Class (MonadIO(..))
 import           Control.Monad.Except (MonadError(..), ExceptT, runExceptT)
 import           Control.Monad.Reader (MonadReader(..), ReaderT(..), asks)
 import qualified Data.Bimap as Bimap
 import qualified Data.BitVector.Sized as BV
 import           Data.Char (digitToInt, isAscii)
+import           Data.Foldable
 import           Data.HashMap.Lazy (HashMap)
 import qualified Data.HashMap.Lazy as HashMap
 import           Data.IORef
+import qualified Data.List as List
 import           Data.Map.Strict (Map)
 import qualified Data.Map.Strict as Map
 import           Data.Monoid
@@ -151,6 +153,7 @@ import qualified What4.Config as CFG
 import qualified What4.Expr.Builder as B
 import           What4.Expr.GroundEval
 import qualified What4.Interface as I
+import           What4.Panic
 import           What4.ProblemFeatures
 import           What4.Protocol.Online
 import           What4.Protocol.ReadDecimal
@@ -984,7 +987,7 @@ parseFn sym nm params_sexp body_sexp = do
       let let_env = HashMap.fromList $ zip nms $ map (mapSome $ I.varExpr sym) vars
       proc_res <- runProcessor (ProcessorEnv { procSym = sym, procLetEnv = let_env }) $ parseExpr sym body_sexp
       Some body_expr <- either fail return proc_res
-      I.SomeSymFn <$> I.definedFn sym (I.safeSymbol $ Text.unpack nm) vars_assign body_expr I.NeverUnfold
+      I.SomeSymFn <$> I.definedFn sym (I.safeSymbol $ Text.unpack nm) vars_assign body_expr I.AlwaysUnfold
 
 parseVar :: I.IsSymExprBuilder sym => sym -> SExp -> IO (Text, Some (I.BoundVar sym))
 parseVar sym sexp = case sexp of
@@ -1065,6 +1068,15 @@ data Op sym where
       Op sym
 
 data Assoc = RightAssoc | LeftAssoc
+  deriving (Eq, Ord, Show)
+
+opAssoc :: Op sym -> Maybe Assoc
+opAssoc = \case
+  Op1{} -> Nothing
+  Op2 _ assoc _ -> assoc
+  BVOp1{} -> Nothing
+  BVOp2 assoc _ -> assoc
+  BVComp2{} -> Nothing
 
 newtype Processor sym a = Processor (ExceptT String (ReaderT (ProcessorEnv sym) IO) a)
   deriving (Functor, Applicative, Monad, MonadIO, MonadError String, MonadReader (ProcessorEnv sym))
@@ -1168,7 +1180,44 @@ parseExpr sym sexp = case sexp of
     BVProof{} <- getBVProof arg1_expr
     BVProof{} <- getBVProof arg2_expr
     liftIO $ Some <$> I.bvConcat sym arg1_expr arg2_expr
+  SApp [SApp ["_", "extract", i_sexp, j_sexp], arg]
+    | Just i <- parseIntSolverValue i_sexp
+    , Just j <- parseIntSolverValue j_sexp -> do
+      let n = i - j + 1
+      Some j_repr <- return $ mkNatRepr $ fromIntegral j
+      Some n_repr <- return $ mkNatRepr $ fromIntegral n
+      Some arg_expr <- parseExpr sym arg
+      BVProof w_repr <- getBVProof arg_expr
+      case (isPosNat n_repr, testLeq (addNat j_repr n_repr) w_repr) of
+        (Just LeqProof, Just LeqProof) ->
+          liftIO $ Some <$> I.bvSelect sym j_repr n_repr arg_expr
+        _ -> throwError ""
   SApp ((SAtom operator) : operands) -> case HashMap.lookup operator (opTable @sym) of
+    -- Sometimes, binary operators can be applied to more than two operands,
+    -- e.g., (+ 1 2 3 4). We want to uniformly represent binary operators such
+    -- that they are always applied to two operands, so this case converts the
+    -- expression above to:
+    --
+    -- - (+ (+ (+ 1 2) 3) 4) (if + is left-associative)
+    -- - (+ 1 (+ 2 (+ 3 4))) (if + is right-associative)
+    --
+    -- We then call `parseExpr` and recurse, which will reach one of the cases
+    -- below.
+    Just op
+      | Just LeftAssoc <- opAssoc op
+      , op1:op2:op3:ops <- operands ->
+          parseExpr sym $ foldl' (\acc arg -> SApp [SAtom operator, acc, arg]) op1 (op2:op3:ops)
+
+        -- For right-associative operators, we could alternatively call
+        -- init/last on the list of operands and call foldr on the results. The
+        -- downside, however, is that init/last are partial functions. To avoid
+        -- this partiality, we instead match on `reverse operands` and call
+        -- foldl on the results (with the order of acc/arg swapped). This
+        -- achieves the same end result and maintains the same asymptotic
+        -- complexity as using init/tail.
+      | Just RightAssoc <- opAssoc op
+      , op1:op2:op3:ops <- List.reverse operands ->
+          parseExpr sym $ foldl' (\acc arg -> SApp [SAtom operator, arg, acc]) op1 (op2:op3:ops)
     Just (Op1 arg_types fn) -> do
       args <- mapM (parseExpr sym) operands
       exprAssignment arg_types args >>= \case
@@ -1209,6 +1258,7 @@ parseExpr sym sexp = case sexp of
                                        (show n)
     _ -> throwError ""
   _ -> throwError ""
+
 -- | Verify a list of arguments has a single argument and
 -- return it, else raise an error.
 readOneArg ::
@@ -1333,8 +1383,10 @@ instance SMTLib2Tweaks a => SMTReadWriter (Writer a) where
           AckUnsat   -> Just $ Unsat ()
           AckUnknown -> Just Unknown
           _ -> Nothing
-    in getLimitedSolverResponse "sat result" satRsp s
-       (head $ reverse $ checkCommands p)
+        cmd = case reverse $ checkCommands p of
+                cmd':_ -> cmd'
+                []     -> panic "smtSatResult" ["Empty list of checkCommands"]
+    in getLimitedSolverResponse "sat result" satRsp s cmd
 
   smtUnsatAssumptionsResult p s =
     let unsatAssumpRsp = \case
@@ -1595,8 +1647,11 @@ startSolver solver ack setup tmout feats strictOpt auxOutput sym = do
 shutdownSolver
   :: SMTLib2GenericSolver a => a -> SolverProcess t (Writer a) -> IO (Exit.ExitCode, Lazy.Text)
 shutdownSolver _solver p = do
-  -- Tell solver to exit
-  writeExit (solverConn p)
+  -- Tell solver to exit, if the process is still running
+  status <- Streams.getProcessExitCode (solverHandle p)
+  case status of
+    Just _ -> return ()
+    Nothing -> writeExit (solverConn p)
   txt <- readAllLines (solverStderr p)
   stopHandleReader (solverStderr p)
   ec <- solverCleanupCallback p
