@@ -63,6 +63,8 @@ allOnlineSolvers =
     ,  AnOnlineSolver @(SMT2.Writer CVC5) Proxy, cvc5Features, cvc5Options, Just cvc5Timeout)
   , (SolverName "Yices"
     , AnOnlineSolver @Yices.Connection Proxy, yicesDefaultFeatures, yicesOptions, Just yicesGoalTimeout)
+  , (SolverName "Bitwuzla"
+    , AnOnlineSolver @(SMT2.Writer Bitwuzla) Proxy, bitwuzlaFeatures, bitwuzlaOptions, Just bitwuzlaTimeout)
   , (SolverName "Boolector"
     , AnOnlineSolver @(SMT2.Writer Boolector) Proxy, boolectorFeatures, boolectorOptions, Just boolectorTimeout)
 #ifdef TEST_STP
@@ -378,6 +380,7 @@ timeoutTests testLevel solvers =
                         , (SolverName "CVC4",       7.5  % Second)    -- CVC4 1.8
                         , (SolverName "CVC5",       0.40  % Second)   -- CVC5 1.0.0
                         , (SolverName "Yices",      2.9  % Second)    -- Yices 2.6.1
+                        , (SolverName "Bitwuzla",   0.51 % Second)    -- Bitwuzla 0.3.0
                         , (SolverName "Boolector",  7.2  % Second)    -- Boolector 3.2.1
                         , (SolverName "STP",        1.35 % Second)    -- STP 2.3.3
                         ]
@@ -419,7 +422,10 @@ timeoutTests testLevel solvers =
                 -- quickly (~0.25s).  This doesn't allow timeout
                 -- testing, and the speed suggests an improper
                 -- result as well.
-                (SolverName "CVC4", SolverVersion v) | "1.7" `elem` words v->
+                (SolverName "CVC4", SolverVersion v) | "1.7" `elem` words v ->
+                  ignoreTestBecause "solver completes too quickly"
+                -- TODO(#278): Maybe the same problem as above?
+                (SolverName "CVC4", SolverVersion v) | "1.8" `elem` words v ->
                   ignoreTestBecause "solver completes too quickly"
                 _ -> id
         in maybeSkipTest $ testGroup (snamestr $ testSolverName sti)
